from typing import Iterable

from icalendar import Calendar  # type: ignore

from .entities import Event
from .event_parser import EventParser
from .fetching import BaseWebFetcher


class ICalFetcher(BaseWebFetcher):
    base_url = "https://ro.umich.edu/calendars/feed"


class ICalEventParser(EventParser):
    def calendar_for_feed(self, feed: str) -> Calendar:
        return Calendar.from_ical(feed)

    def events_for_calendar(self, calendar: Calendar) -> Iterable[Event]:
        for subcomponent in calendar.subcomponents:
            yield Event(
                str(subcomponent["DESCRIPTION"]),
                subcomponent["DTSTART"].dt,
                subcomponent["DTEND"].dt,
            )

    def events(self, term_name: str) -> Iterable[Event]:
        feed = self.fetcher.fetch_data(term_name)
        calendar = self.calendar_for_feed(feed)
        return self.events_for_calendar(calendar)
